;  Basic_FREQSOURCE.ASM.      G4JNt   JAn 2011

;32 bit / 8 bit DDS for Audio generation   
;Interrupt runs at Fosc / 200 (Fclock / 50).  /2 Prescale, int counts 25 
;  Increments 32 bit counterD0-3   with value in F0-3.
;  MS Byte used as entry to 8 bit sine lookup table
;This version has just the code for the DDS as demo.

    LIST P = 16F628
    INCLUDE "P16F628.INC"
    __config    0x3F62      ;LV Programming off , OSC HS Mode   ,  works fwith ext input OK
    __idlocs    0x8131		

	errorlevel 2		;Prevents annpyoing messages

    org	0x2100	;EEPROM Data
FREQ
	de	0x33, 0x33, 0x33, 0x33		;0x33333333  =   1/5 Fc for testing




	#define		TESTSTRB	PORTA, 0

    org     0
	goto    startup         ;jump to main code

;============== Interrupt routine, get here 4 cycles after Timer overflow =============
    org 4

ints			;Using  divide by  prescalar,  Interupt service runs at at Fosc / 200
			; when set to count 25 before overflowing, ie  every 50 clocks = sampling rate

     	bcf     INTCON , T0IF   ;reset the interrupt flag
        bsf	TESTSTRB        ;Spare instruction needed to make up 8 clocks total, up to the point of reloading TMR0
				;So use it to pulse a test point
        movlw   d'235'          ; Load in value to count up to 256 FROM (= NNN) 
        movwf   TMR0            ;Timer restarts counting 2 cycles after here.
                                ;Up to this point takes 8 clocks from Interrupt trigger
                                ;So NNN = 256 - [Wanted count] + 8 / Prescale  
				;  ie =   256 - 25 + 8 / 2 = 235
 

 ;Below here is not time-critical, but must be less than than about 42 clocks total
 ;   so the whole interrupt takes < 50 clocks and doesn't overrun.

	movf	D3, W		;First output the D/A value from the LAST pass though so there is
	call	SineTable	;   no jittering due to variable timing of the addition routine
	movwf	PORTB

	movf	F0, W		;Now add the frequency data stored in F3/2/1/0 to D3/2/1/0
	addwf	D0
	btfss	STATUS, C	;Add lowest order byte and check for and carry at each stage
	goto	Add1Done	;  that could ripple right through to MS byte
	incf	D1
	btfsc	STATUS, Z
	incf	D2
	btfsc	STATUS, Z
	incf	D3
Add1Done
	movf	F1, W		;Add the next byte
	addwf	D1
	btfss	STATUS, C
	goto	Add2Done
	incf	D2
	btfsc	STATUS, Z

	incf	D3
Add2Done			
	movf	F2, W		;... and the next
	addwf	D2
	btfsc	STATUS, C
	incf	D3	

	movf	F3, W		;MS Byte now added in.  If it has overflowed, its is exactly what we want
	addwf	D3		;D0-3 are now updated ready for next pass through, with D3 only sent to D/A converter.

OutInt
	bcf	TESTSTRB	;Reset the Test Point	
	retfie			;This  interrupt takes a max of 35 clocks for worst case addition, so does leave 
				;  a bit of scope for extra tasks or faster sampling.

;===================================== End of Interrupt routine ====================================

startup 
	clrf    PORTA       	;Needed on 16F628; a special setup to disable the comparator and allow PORTA
	movlw   7		;  to work properly.  Possibly a c__k-up by Microchip ?
	movwf   CMCON
	
	bsf     STATUS,RP0      ;ram page 1

        movlw   b'10000000'     ;Set timer for clocking from internal  processor clock, with a Prescalar /2
        movwf   OPTION_REG

	movlw   b'00000110'     ;Only port A0 is used in this version, for the test point.
	movwf   PORTA           ;
	movlw   b'00000000'     ;Set all as outputs on PORTB, to go to D/A R-2R ladder
	movwf   PORTB           ;

	bcf	STATUS, RP0

	movlw	HIGH(TabContents)	;Get the High order byte of start of sine table and place 
	movwf	PCLATH		; into the holding register, ready for 'computed goto' in table lookup

	call	LoadFreq	;Get frequency data stored in EE memory into the working F0-F3 registers

	bsf     INTCON, T0IE    ;Enable the timer interrupt
	bsf	INTCON, GIE	;  and set it running
;------------------
MainLoop
	nop			;Just sit here, doing nothing and waiting for the next interrupt....
	goto	MainLoop	;Can't use sleep, as oscillator must still run to generate the interrupt

;============================= Subroutines below here ========================
LoadFreq
	movlw	LOW(FREQ)	;Stored in EE high byte first to help readability
	call	GetEE
	movwf	F3

	movlw	LOW(FREQ + 1)
	call	GetEE
	movwf	F2

	movlw	LOW(FREQ + 2)
	call	GetEE
	movwf	F1

	movlw	LOW(FREQ + 3)
	call	GetEE
	movwf	F0

	return
;------------------------
GetEE 
    bsf     STATUS,RP0      ;ram page 1
    movwf   EEADR
    bsf     EECON1 , RD
    movf    EEDATA , W
    bcf     STATUS , RP0    ;ram page 0
    return

;=====================  End of Subroutines ===============================
 ;Place the SINE table in an area of memory well away from any later prog enhancements

  org 0x6FF    			; As the table is 256 long, this computed goto must sit immediately 
				;   before the first entry
SineTable
			;Ensure  TabContents falls EXACTLY on the next page boundary, here 0x700.
	addwf	PCL	;   double check in .LST file  after assembly, preceded by addwf command
TabContents
    dt    0x80,0x83,0x86,0x89,0x8C,0x8F,0x92,0x95	;One complete sinewave cycle, 256 entries
    dt    0x98,0x9C,0x9F,0xA2,0xA5,0xA8,0xAB,0xAE
    dt    0xB0,0xB3,0xB6,0xB9,0xBC,0xBF,0xC1,0xC4
    dt    0xC7,0xC9,0xCC,0xCE,0xD1,0xD3,0xD5,0xD8
    dt    0xDA,0xDC,0xDE,0xE0,0xE2,0xE4,0xE6,0xE8
    dt    0xEA,0xEC,0xED,0xEF,0xF0,0xF2,0xF3,0xF5
    dt    0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFC
    dt    0xFD,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF
    dt    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFE,0xFE
    dt    0xFD,0xFC,0xFC,0xFB,0xFA,0xF9,0xF8,0xF7
    dt    0xF6,0xF5,0xF3,0xF2,0xF0,0xEF,0xED,0xEC
    dt    0xEA,0xE8,0xE6,0xE4,0xE2,0xE0,0xDE,0xDC
    dt    0xDA,0xD8,0xD5,0xD3,0xD1,0xCE,0xCC,0xC9
    dt    0xC7,0xC4,0xC1,0xBF,0xBC,0xB9,0xB6,0xB3
    dt    0xB0,0xAE,0xAB,0xA8,0xA5,0xA2,0x9F,0x9C
    dt    0x98,0x95,0x92,0x8F,0x8C,0x89,0x86,0x83
    dt    0x7F,0x7C,0x79,0x76,0x73,0x70,0x6D,0x6A
    dt    0x67,0x63,0x60,0x5D,0x5A,0x57,0x54,0x51
    dt    0x4F,0x4C,0x49,0x46,0x43,0x40,0x3E,0x3B
    dt    0x38,0x36,0x33,0x31,0x2E,0x2C,0x2A,0x27
    dt    0x25,0x23,0x21,0x1F,0x1D,0x1B,0x19,0x17
    dt    0x15,0x13,0x12,0x10,0x0F,0x0D,0x0C,0x0A
    dt    0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x03
    dt    0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00
    dt    0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01
    dt    0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x08
    dt    0x09,0x0A,0x0C,0x0D,0x0F,0x10,0x12,0x13
    dt    0x15,0x17,0x19,0x1B,0x1D,0x1F,0x21,0x23
    dt    0x25,0x27,0x2A,0x2C,0x2E,0x31,0x33,0x36
    dt    0x38,0x3B,0x3E,0x40,0x43,0x46,0x49,0x4C
    dt    0x4F,0x51,0x54,0x57,0x5A,0x5D,0x60,0x63
    dt    0x67,0x6A,0x6D,0x70,0x73,0x76,0x79,0x7C

;--------------------------------
    Cblock 0x20
	Temp
	DelCount
	RxChar
	BaudConst
	Counter
	AutoEEADR
	D0, D1, D2, D3
	F0, F1, F2, F3
	Stack0, Stack1,Stack2, Stack3,Stack4, Stack5,Stack6, Stack7
	Stack8, Stack9,StackA, StackB,StackC, StackD,StackE, StackF
    endc

	end















