;  COHFSKSRC.ASM.      G4JNT   Feb 2011

;32 bit / 8 bit DDS for Audio generation   
;Interrupt runs at Fosc / 160 (Fclock / 40).  /2 Prescale, int counts 20
;  Increments two 32 bit counters D0-3 and E0-3   with values in F0-3 7 G0-3
;  MS Byte used as entry to 8 bit sine lookup table, selected by SERIALIN line
;User programmable via RS232 with auto baud detect
; 2 tone FSK selected from serial input line, switched between two
; simultaneous NCOs, maintaining phase coherence on each

    LIST P = 16F628
    INCLUDE "P16F628.INC"
    __config    0x3F62      ;LV Programming off , OSC HS Mode   use 3F63 for ext source

    __idlocs    0x8130

	errorlevel 2

    org	0x2100
FREQ
	de	0x33, 0x33, 0x33, 0x33		;Mark,  or '0'  freq
	de	0x11, 0x11, 0x11, 0x11		;Space, or '1' freq

STOREDBAUD
	de	d'64'		;Correct value for 10MHz clock


	#define		TESTSTRB	PORTA, 0
	#define		KEEPBAUD	PORTA, 1
	#define		SERIALIN	PORTA, 2
	#define		SERIALOUT	PORTA, 3

	#define		MKSP		Flags, 0

    org     0

	clrf	INTCON          ;
	goto    startup         ;jump to main code
;------------------ Interrupt routine, get here 4 cycles after TMR overflow --
    org 4
ints      	  ;Interupt service at Fosc / 256.  NO REGISTERS are saved,

     	bcf     INTCON , T0IF   ;  every 64 clocks = sampling rate
        bsf	TESTSTRB        ;Spare instruction needed to make up 8 clocks total up to point of reloading TMR0
        movlw   d'228'          ; = NNN:  Timer counts 32 of (Fc / 8)
        movwf   TMR0            ;Timer restarts counting 2 cycles after here.
                                ;To here takes 8 clocks (with the extra instruction)
                                ;NNN = 256 - Count + 8/Prescale  = 256 - 32 + 8/2 = 196

 ;Below here is not time-critical, but must be less than than about 56 clocks total
 ;    to not overrun the whole interrupt service.

	movf	D3, W		;Do this first so there is no jittering due to variable add routine times.
	btfsc	SERIALIN
	movf	E3, W
	call	SineTable	;Takes 5 clocks incl call and return
	movwf	PORTB

	movf	F0, W
	addwf	D0
	btfss	STATUS, C
	goto	Add1Done
	incf	D1
	btfsc	STATUS, Z
	incf	D2
	btfsc	STATUS, Z
	incf	D3
Add1Done
	movf	F1, W
	addwf	D1
	btfss	STATUS, C
	goto	Add2Done
	incf	D2
	btfsc	STATUS, Z

	incf	D3
Add2Done
	movf	F2, W
	addwf	D2
	btfsc	STATUS, C
	incf	D3	

	movf	F3, W
	addwf	D3			;D0-3 updated ready for next pass through

	movf	G0, W			;Now do the other accumulator
	addwf	E0
	btfss	STATUS, C
	goto	Add3Done
	incf	E1
	btfsc	STATUS, Z
	incf	E2
	btfsc	STATUS, Z
	incf	E3
Add3Done
	movf	G1, W
	addwf	E1
	btfss	STATUS, C
	goto	Add4Done
	incf	E2
	btfsc	STATUS, Z

	incf	E3
Add4Done
	movf	G2, W
	addwf	E2
	btfsc	STATUS, C
	incf	E3	

	movf	G3, W
	addwf	E3			;E0-3 updated ready for next pass through


	bcf	TESTSTRB		;This lot max of 62 clocks 

	retfie

;=====================================
startup 
	clrf    PORTA       ;Special setup to disable comparator on 'F628
	movlw   7
	movwf   CMCON
	
	bsf     STATUS,RP0      ;ram page 1

        movlw   b'10000000'     ;Internal input, Prescalar /2 no pull-ups
        movwf   OPTION_REG

	movlw   b'00000110'     ;
	movwf   PORTA           ;	A2 is serial in, A1 Link to measure baud rate
	movlw   b'00000000'     ;
	movwf   PORTB           ;

	bcf	STATUS, RP0

	movlw	HIGH(TabContents)
	movwf	PCLATH
	clrf	FSKOffset

	movlw	LOW(STOREDBAUD)	;
	call	GetEE
	movwf	BaudConst

	btfss	KEEPBAUD		;If if link is in place, goto user setup
	goto	UserSetup
Resume
	call	ShowFreq


	bsf     INTCON, T0IE   
	bsf	INTCON, GIE
;------------------
MainLoop			
	nop
	goto	MainLoop
;------------------
UserSetup			

	clrf	Counter		;Wait for a start bit and measure its length
	clrf	DelCount	;Use these as a 16 bit counter
FirstEdge
	btfss	SERIALIN	;Look for a rising edge	
	goto	FirstEdge		

SecondEdge
	incf	Counter
	btfsc	STATUS, Z	;This loop is 8 long +2 , so Delcount/Counter contains [no. of clocks in Start bit - 2] / 8.
	incf	DelCount	;     Eg.  for 20MHz osc, Fc = 5MHz, so count = 520
	nop
	nop			;Baud delay = (16 * BaudConst + 8) * 2 + 8   =   32 * BaudConst + 24
	btfsc	SERIALIN	;Result from above, 'N' is clocks/8 in a complete bit interval
	goto	SecondEdge	
				
	rrf	DelCount	;   So 8.N + 2 = 32.baudConst + 24,   or N = 2.BaudConst + [2.75]
	rrf	Counter		;  BaudConst ~=   N/4 - 1 for a Max Fosc of 39.28MHz
	rrf	DelCount
	rrf	Counter
	decf	Counter, W
	movwf	BaudConst

	call	Msg2	
	movf	BaudConst, W
	call	Hex232

	call	CrLf

	movlw	LOW(STOREDBAUD)
	movwf	AutoEEADR
	movf	BaudConst, W
	call	StoreEEW

	movlw	LOW(STOREDBAUD)
	call	GetEE
	movwf	BaudConst

	clrf	FSKOffset	;Get 4 bytes of data from ser input and store in Mark 
	call	MarkMsg		;  frequency area
	call	GetData

	movlw	4
	movwf	FSKOffset	;Get 4 bytes and store in Space area
	call	SpaceMsg
	call	GetData

	goto	Resume

;==================================================== Subroutines below here
ShowFreq
	movlw	"'"
	call	Send232
	movlw	"0"
	call	Send232
	movlw	"'"
	call	Send232
	movlw	"-"
	call	Send232
	

	movlw	LOW(FREQ)
	call	GetEE
	movwf	F3
	call	Hex232

	movlw	LOW(FREQ + 1)
	call	GetEE
	movwf	F2
	call	Hex232

	movlw	LOW(FREQ + 2)
	call	GetEE
	movwf	F1
	call	Hex232

	movlw	LOW(FREQ + 3)
	call	GetEE
	movwf	F0
	call	Hex232

	call	CrLf

	movlw	"'"
	call	Send232
	movlw	"1"
	call	Send232
	movlw	"'"
	call	Send232
	movlw	"-"
	call	Send232

	movlw	LOW(FREQ + 4)
	call	GetEE
	movwf	G3
	call	Hex232

	movlw	LOW(FREQ + 5)
	call	GetEE
	movwf	G2
	call	Hex232

	movlw	LOW(FREQ + 6)
	call	GetEE
	movwf	G1
	call	Hex232

	movlw	LOW(FREQ + 7)
	call	GetEE
	movwf	G0
	call	Hex232

	call	CrLf

	return

;--------------------------------
GetData				;Read 8 hex characters from RS232
	call	PushStack	
	call	WaitData
	movwf	Stack1

	call	Send232

	movf	Stack1, W
	sublw	0x0D		;[cr] terminator
	btfss   STATUS, Z
	goto	GetData

;###################################   DEBUG   ####################
	movf	Stack9, W	
	call	Send232
	movf	Stack8, W
	call	Send232
	movf	Stack7, W
	call	Send232
	movf	Stack6, W
	call	Send232
	movf	Stack5, W
	call	Send232
	movf	Stack4, W
	call	Send232
	movf	Stack3, W
	call	Send232
	movf	Stack2, W
	call	Send232
	movf	Stack1, W
	call	Send232
 ;###################################   DEBUG   ####################
				;Stack contents  EDCBA98765432  1
    movf    Stack9, W		;  D registers   ~~~4433221100[cr]
    call    DecodeHex		
    movwf   D3			;  transfer binary data to 3/2/1/0	
    swapf   D3
    movf    Stack8, W	
    call    DecodeHex
    addwf   D3

    movf    Stack7, W
    call    DecodeHex
    movwf   D2
    swapf   D2
    movf    Stack6, W
    call    DecodeHex
    addwf   D2

    movf    Stack5, W
    call    DecodeHex
    movwf   D1
    swapf   D1
    movf    Stack4, W
    call    DecodeHex
    addwf   D1

    movf    Stack3, W
    call    DecodeHex
    movwf   D0
    swapf   D0
    movf    Stack2, W
    call    DecodeHex
    addwf   D0
	
	call	CrLf
	movlw	"-"
	call	Send232
	call	CrLf
	
	movlw	LOW(FREQ)
	addwf	FSKOffset, W
	movwf	AutoEEADR
	movf	D3, W
	call	StoreEEW

	movlw	LOW(FREQ + 1)
	addwf	FSKOffset, W
	movwf	AutoEEADR
	movf	D2, W
	call	StoreEEW

	movlw	LOW(FREQ + 2)
	addwf	FSKOffset, W
	movwf	AutoEEADR
	movf	D1, W
	call	StoreEEW

	movlw	LOW(FREQ + 3)
	addwf	FSKOffset, W
	movwf	AutoEEADR
	movf	D0, W
	call	StoreEEW

	return
;--------------------------------
WaitData        ;Get character from RS232 input and convert to upper case
    btfss   SERIALIN         ;look for start transition, low/high = 1/0
    goto    WaitData    ;

    call    HalfBitDelay

    btfss   SERIALIN         ;make sure it's still high, now centre of bit
    goto    WaitData      ;
    movlw   8           ;
    movwf   Counter
    clrf    RxChar     
ByteLoop                
    call    HalfBitDelay
    call    HalfBitDelay
    nop
    bcf     STATUS , C     
    btfss   SERIALIN         
    bsf     STATUS , C     
    rrf     RxChar
    decfsz  Counter
    goto    ByteLoop      ;this loop BitDelay + 8

    call    HalfBitDelay    ;Wait for some stop bit delay
    call    HalfBitDelay

    movlw   "a"             ;Convert to Upper case
    subwf   RxChar, W       ;W = Char - "a"
    btfss   STATUS, C       ;if-ve (C=0) char < "a" so ignore
    goto    UpCaseDone

    movf    RxChar, W
    sublw   "z"             ;W = "z" - Char
    btfss   STATUS, C       ;if -ve (C=0)  char > "z" so ignore
    goto    UpCaseDone

    bcf     RxChar, 5       ;Reset bit 5 to force upper case
UpCaseDone
    movf    RxChar , W    
    return                  ;received byte in RxChar and W
;-------------------------
Hex232      ;Send a byte as hex data. Uses   RxChar [Temp, Counter, DelCount]
    movwf   RxChar          ;Use RxChar as temporary storage only
    swapf   RxChar, W       ;Send MS Nibble
    andlw   0x0F
    call    HexToAscii      
    call    Send232
    movf    RxChar, W       ;Send LS Nibble
    andlw   0x0F
    call    HexToAscii
    call    Send232
    return
;-------------------------
Send232                         ;Enter with data in W, send it to PC
    movwf   Temp            	;Uses Temp, Counter, [DelCount]
    bsf     SERIALOUT         	;start bit
    call    HalfBitDelay    
    call    HalfBitDelay    
    movlw   8
    movwf   Counter
SendDatLoop
    rrf     Temp        		;bit into C
    btfsc   STATUS , C
    bcf     SERIALOUT
    btfss   STATUS , C
    bsf     SERIALOUT     
    call    HalfBitDelay
    call    HalfBitDelay
    decfsz  Counter
    goto    SendDatLoop     		;loop 8 + BitDelay long
    bcf     SERIALOUT     ;stop bit
    call    HalfBitDelay
    call    HalfBitDelay
    call    HalfBitDelay
    call    HalfBitDelay

    return
;--------------------------------------
HalfBitDelay        			;Total delay 16.N + 8  including call and return,
	movf    BaudConst, W        ;RS232 routines call this twice, plus 8 clocks overhead 
	movwf   DelCount      	
LoopBit
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	goto	$+1
	nop
	decfsz  DelCount        
	goto    LoopBit

	goto	$+1
	nop
	return
;-------------------------
DecodeHex           ;Single ASCII character to binary 
    movwf   Temp            ;Uses Temp
    movlw   0x30
    subwf   Temp            ;subtract $30 and leave in Temp
    btfss   STATUS , C      ;if < $30, result -ve so C=0, error, set=0
    clrf    Temp

    movf    Temp , W        
    sublw   d'9'            ;W = 9 - W
    btfsc   STATUS , C      ;if -ve, C=0, char > 9, so subtract 7
    goto    ConvDone
    movlw   d'7'
    subwf   Temp            ;
ConvDone
    movf    Temp , W        ;
    andlw   0x0F            ;mask out any rubbish left in
    return              
;-------------------------
HexToAscii          ;Uses Temp
    addlw   0x30
    movwf   Temp
    sublw   0x39
    btfsc   STATUS , C
    goto    AsciiDone
    movlw   7
    addwf   Temp
AsciiDone
    movf    Temp , W
    return
;-------------------------
;------------------------
GetEE 
    bsf     STATUS,RP0      ;ram page 1
    movwf   EEADR
    bsf     EECON1 , RD
    movf    EEDATA , W
    bcf     STATUS , RP0    ;ram page 0
    return
;-------------------------
StoreEEW          ;Enter with W = EE data,  AutoEEADR = EE address
    bsf     STATUS, RP0
    movwf   EEDATA
    bcf     STATUS, RP0
    movf    AutoEEADR, W
    bsf     STATUS, RP0
    movwf   EEADR    

    bsf     STATUS , RP0    ;All EE registers in Bank 1

    bsf     EECON1 , WREN   ;Enable EEprom writing
    movlw   0x55
    movwf   EECON2
    movlw   0xAA
    movwf   EECON2
    bsf     EECON1 , WR

    bcf     STATUS, RP0     ;PIR1 register is in bank 0
EEwaitW
   
    btfss   PIR1 , EEIF
    goto    EEwaitW
    bcf     PIR1 , EEIF       ;has to be cleared manually

    bsf     STATUS, RP0
    bcf     EECON1 , WR
    bcf     STATUS , RP0

    return
;-------------------------
SpaceMsg
	movlw	"S"
	call	Send232
	movlw	"p"
	call	Send232
	movlw	"a"
	call	Send232
	movlw	"c"
	call	Send232
	movlw	"e"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"F"
	call	Send232
	movlw	"r"
	call	Send232
	movlw	"e
	call	Send232
	movlw	"q"
	call	Send232
	
	goto	Freq2Msg
;.......
MarkMsg
	movlw	"G"
	call	Send232
	movlw	"4"
	call	Send232
	movlw	"J"
	call	Send232
	movlw	"N"
	call	Send232
	movlw	"T"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"P"
	call	Send232
	movlw	"I"
	call	Send232
	movlw	"C"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"D"
	call	Send232
	movlw	"D"
	call	Send232
	movlw	"S"
	call	Send232
	movlw	"."
	call	Send232
	call	CrLf

	movlw	"M"
	call	Send232
	movlw	"a"
	call	Send232
	movlw	"r"
	call	Send232
	movlw	"k"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"F"
	call	Send232
	movlw	"r"
	call	Send232
	movlw	"e"
	call	Send232
	movlw	"q"
	call	Send232
	
Freq2Msg
	call	CrLf
	movlw	"E"
	call	Send232
	movlw	"n"
	call	Send232
	movlw	"t"
	call	Send232
	movlw	"e"
	call	Send232
	movlw	"r"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"8"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"H"
	call	Send232
	movlw	"e"
	call	Send232
	movlw	"x"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"C"
	call	Send232
	movlw	"h"
	call	Send232
	movlw	"a"
	call	Send232
	movlw	"r"
	call	Send232
	movlw	"s"
	call	Send232
	movlw	"."
	call	Send232
	movlw	"."
	call	Send232
	movlw	"."
	call	Send232
	call	CrLf
	return	
;------------------------
Msg2
	movlw	"A"
	call	Send232
	movlw	"u"
	call	Send232
	movlw	"t"
	call	Send232
	movlw	"o"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"B"
	call	Send232
	movlw	"a"
	call	Send232
	movlw	"u"
	call	Send232
	movlw	"d"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"s"
	call	Send232
	movlw	"e"
	call	Send232
	movlw	"t"
	call	Send232
	movlw	" "
	call	Send232

	movlw	"C"
	call	Send232
	movlw	"o"
	call	Send232
	movlw	"n"
	call	Send232
	movlw	"s"
	call	Send232
	movlw	"t"
	call	Send232
	movlw	" "
	call	Send232
	movlw	"0"
	call	Send232
	movlw	"x"
	call	Send232

	return

;------------------------
PushStack               ;Free up Stack1,  get rid of oldest
    movf    StackE, W
    movwf   StackF
    movf    StackD, W
    movwf   StackE
    movf    StackC, W
    movwf   StackD
    movf    StackB, W
    movwf   StackC
    movf    StackA, W
    movwf   StackB
    movf    Stack9, W
    movwf   StackA
    movf    Stack8, W
    movwf   Stack9
    movf    Stack7, W
    movwf   Stack8
    movf    Stack6, W
    movwf   Stack7
    movf    Stack5, W
    movwf   Stack6
    movf    Stack4, W
    movwf   Stack5
    movf    Stack3, W
    movwf   Stack4   
    movf    Stack2, W
    movwf   Stack3
    movf    Stack1, W
    movwf   Stack2
    return
;-----------------------
ClrStack
    clrf    StackF
    clrf    StackE
    clrf    StackD
    clrf    StackC
    clrf    StackB
    clrf    StackA
    clrf    Stack9
    clrf    Stack8
    clrf    Stack7
    clrf    Stack6
    clrf    Stack5
    clrf    Stack4
    clrf    Stack3
    clrf    Stack2
    clrf    Stack1
    return
;-----------------------    
CrLf
    movlw   0x0D
    call    Send232
    movlw   0x0A
    call    Send232
    return
;-----------------------
;---------------------

  org 0x6FF
SineTable
		;Adjust so TabContents falls EXACTLY on the next page boundary
	addwf	PCL
TabContents
    dt    0x80,0x83,0x86,0x89,0x8C,0x8F,0x92,0x95
    dt    0x98,0x9C,0x9F,0xA2,0xA5,0xA8,0xAB,0xAE
    dt    0xB0,0xB3,0xB6,0xB9,0xBC,0xBF,0xC1,0xC4
    dt    0xC7,0xC9,0xCC,0xCE,0xD1,0xD3,0xD5,0xD8
    dt    0xDA,0xDC,0xDE,0xE0,0xE2,0xE4,0xE6,0xE8
    dt    0xEA,0xEC,0xED,0xEF,0xF0,0xF2,0xF3,0xF5
    dt    0xF6,0xF7,0xF8,0xF9,0xFA,0xFB,0xFC,0xFC
    dt    0xFD,0xFE,0xFE,0xFF,0xFF,0xFF,0xFF,0xFF
    dt    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF,0xFE,0xFE
    dt    0xFD,0xFC,0xFC,0xFB,0xFA,0xF9,0xF8,0xF7
    dt    0xF6,0xF5,0xF3,0xF2,0xF0,0xEF,0xED,0xEC
    dt    0xEA,0xE8,0xE6,0xE4,0xE2,0xE0,0xDE,0xDC
    dt    0xDA,0xD8,0xD5,0xD3,0xD1,0xCE,0xCC,0xC9
    dt    0xC7,0xC4,0xC1,0xBF,0xBC,0xB9,0xB6,0xB3
    dt    0xB0,0xAE,0xAB,0xA8,0xA5,0xA2,0x9F,0x9C
    dt    0x98,0x95,0x92,0x8F,0x8C,0x89,0x86,0x83
    dt    0x7F,0x7C,0x79,0x76,0x73,0x70,0x6D,0x6A
    dt    0x67,0x63,0x60,0x5D,0x5A,0x57,0x54,0x51
    dt    0x4F,0x4C,0x49,0x46,0x43,0x40,0x3E,0x3B
    dt    0x38,0x36,0x33,0x31,0x2E,0x2C,0x2A,0x27
    dt    0x25,0x23,0x21,0x1F,0x1D,0x1B,0x19,0x17
    dt    0x15,0x13,0x12,0x10,0x0F,0x0D,0x0C,0x0A
    dt    0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x03
    dt    0x02,0x01,0x01,0x00,0x00,0x00,0x00,0x00
    dt    0x00,0x00,0x00,0x00,0x00,0x00,0x01,0x01
    dt    0x02,0x03,0x03,0x04,0x05,0x06,0x07,0x08
    dt    0x09,0x0A,0x0C,0x0D,0x0F,0x10,0x12,0x13
    dt    0x15,0x17,0x19,0x1B,0x1D,0x1F,0x21,0x23
    dt    0x25,0x27,0x2A,0x2C,0x2E,0x31,0x33,0x36
    dt    0x38,0x3B,0x3E,0x40,0x43,0x46,0x49,0x4C
    dt    0x4F,0x51,0x54,0x57,0x5A,0x5D,0x60,0x63
    dt    0x67,0x6A,0x6D,0x70,0x73,0x76,0x79,0x7C

;--------------------------------
    Cblock 0x20
	Temp
	Flags
	DelCount
	RxChar
	BaudConst
	Counter
	AutoEEADR
	FSKOffset
	D0, D1, D2, D3
	E0, E1, E2, E3
	Stack0, Stack1,Stack2, Stack3,Stack4, Stack5,Stack6, Stack7
	Stack8, Stack9,StackA, StackB,StackC, StackD,StackE, StackF
    endc

    Cblock 0x70
	F0, F1, F2, F3		;These are in an area common to bank 0 and bank 1
	G0, G1, G2, G3
    endc

	end















